@@ -25,6 +25,9 @@
 #include <libgnome/libgnome.h>
 
 #include "rdf.h"
+#include "trans.h"
+
+extern ComplainFunc complain;
 
 typedef enum {
   PKG_NOT_TESTED=0, /* no tests have been performed */
@@ -46,11 +49,34 @@ struct _DepsCheck {
 
 /* do a check to see if a particular requirement is provided localy */
 static gboolean haveLocalProvide(rpmdb db, char *name) {
+#ifndef	HAVE_RPM_4_0
   dbiIndexSet matches;
+#endif
 
   if (name[0] == '/' && g_file_exists(name))
     return TRUE;
-  else if (rpmdbFindByProvides(db, name, &matches) == 0) {
+  else
+#ifdef	HAVE_RPM_4_0
+  { rpmdbMatchIterator mi;
+    Header h;
+
+    mi = rpmdbInitIterator(db, RPMTAG_PROVIDENAME, name, 0);
+    while ((h = rpmdbNextIterator(mi)) != NULL) {
+      rpmdbFreeIterator(mi);
+      return TRUE;
+    }
+    rpmdbFreeIterator(mi);
+    
+    mi = rpmdbInitIterator(db, RPMDBI_LABEL, name, 0);
+    while ((h = rpmdbNextIterator(mi)) != NULL) {
+      rpmdbFreeIterator(mi);
+      return TRUE;
+    }
+    rpmdbFreeIterator(mi);
+
+  }
+#else	/* HAVE_RPM_4_0 */
+  if (rpmdbFindByProvides(db, name, &matches) == 0) {
     int i;
 
     for (i = 0; i < matches.count; i++) {
@@ -67,7 +93,7 @@ static gboolean haveLocalProvide(rpmdb db, char *name) {
     if (res == 1) {
       /* nothing */
     } else if (res == 2)
-      g_warning("Error looking up rpm database for %s", name);
+      complain(_("Error looking up rpm database for %s"), name);
     else {
       int i;
       
@@ -82,6 +108,7 @@ static gboolean haveLocalProvide(rpmdb db, char *name) {
       dbiFreeIndexRecord(matches);
     }
   }
+#endif	/* HAVE_RPM_4_0 */
   return FALSE;
 }
 
