@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <time.h>
+#include <stdarg.h>
 
 #include <gnome.h>
 #include <ghttp.h>
@@ -37,6 +38,12 @@
 UrlStartFunc downloadStart = NULL;
 UrlProgressFunc downloadProgress = NULL;
 UrlDoneFunc downloadDone = NULL;
+int downloadAbort = 0;
+ComplainFunc complain = NULL;
+
+void set_complain_func(ComplainFunc func) {
+	complain = func;
+}
 
 void url_set_callbacks(UrlStartFunc start, UrlProgressFunc progress,
 		       UrlDoneFunc done) {
@@ -92,7 +99,7 @@ static int http_transfer(char *url, FILE *file, char *proxy) {
   if (!request)
     goto err_occured;
   if (proxy && ghttp_set_proxy(request, proxy) != 0) {
-    g_warning("setting of proxy failed");
+    complain(_("Setting of proxy failed"));
     g_free(proxy);
     goto err_occured;
   }
@@ -105,7 +112,7 @@ static int http_transfer(char *url, FILE *file, char *proxy) {
       for (i = 0; pass[i] != '\0'; i++)
 	pass[i] ^= 0x42;
     if (user[0] != '\0' && ghttp_set_proxy_authinfo(request, user, pass)) {
-      g_warning("setting proxy user/password pair failed");
+      complain(_("Setting proxy user/password pair failed"));
       g_free(user);
       g_free(pass);
       goto err_occured;
@@ -114,31 +121,37 @@ static int http_transfer(char *url, FILE *file, char *proxy) {
     g_free(pass);
   }
   if (ghttp_set_uri(request, url) != 0) {
-    g_warning("setting of uri failed");
+    complain(_("Setting of uri failed"));
     goto err_occured;
   }
   ghttp_set_header(request, http_hdr_Connection, "close");
   ghttp_set_header(request, http_hdr_User_Agent, USER_AGENT_STRING);
   if (ghttp_prepare(request) != 0) {
-    g_warning("preparing request failed");
+    complain(_("Preparing request failed"));
     goto err_occured;
   }
   if (ghttp_set_sync(request, ghttp_async)) {
-    g_warning("couldn't set async mode");
+    complain(_("Couldn't set async mode"));
     goto err_occured;
   }
   
   while ((status = ghttp_process(request)) == ghttp_not_done) {
     ghttp_current_status curStat = ghttp_get_status(request);
-    if (downloadProgress)
+    if (downloadProgress && !downloadAbort)
       (*downloadProgress)(curStat.bytes_read, curStat.bytes_total);
+    if (downloadAbort)
+    {
+	complain(_("User aborted transfer"));
+	status = ghttp_error;
+        break;
+    }
   }
   if (status == ghttp_error) {
-    g_warning("processing request failed");
+    complain(_("Processing request failed"));
     goto err_occured;
   }
   if (ghttp_status_code(request) != 200) {
-    g_warning("HTTP error: %d %s", ghttp_status_code(request),
+    complain(_("HTTP error: %d %s"), ghttp_status_code(request),
 	      ghttp_reason_phrase(request));
     goto err_occured;
   }
@@ -161,14 +174,14 @@ static int ftp_transfer(char *host, int port, char *path, FILE *file) {
 
   ftpconn = ftpOpen(host, "anonymous", "gnorpm@", NULL, port);
   if (ftpconn < 0) {
-    g_warning("Couldn't connect to FTP server: %s", ftpStrerror(ftpconn));
+    complain(_("Couldn't connect to FTP server: %s"), ftpStrerror(ftpconn));
     return -1;
   }
   fd = fileno(file);
 
   res = ftpGetFile(ftpconn, path, fd);
   if (res < 0) {
-    g_warning("FTP transfer failed: %s", ftpStrerror(ftpconn));
+    complain(_("FTP transfer failed: %s"), ftpStrerror(ftpconn));
     res = -1;
   }
 
@@ -180,13 +193,23 @@ static int ftp_transfer(char *host, int port, char *path, FILE *file) {
 int url_get_to_file(char *url, FILE *file) {
   char *prot, *host, *path;
   int res = -1, port;
+  static int busy=0;
 
-  g_message("downloading %s", url);
+  complain(_("Downloading %s"), url);
+
+  if(busy)
+  {
+	complain(_("Already busy doing a transfer"));
+	return -1;
+  }
+  busy=1;
+  downloadAbort=0;
 
   if (downloadStart)
     (*downloadStart)(url);
   if (!url_split(url, &prot, &host, &port, &path)) {
-    g_warning("invalid URL");
+    complain(_("Invalid URL"));
+    busy=0;
     return -1;
   }
   if (!g_strcasecmp(prot, "http")) {
@@ -209,21 +232,28 @@ int url_get_to_file(char *url, FILE *file) {
     else
       res = ftp_transfer(host, port, path, file);
   } else
-    g_warning("Unsupported URL format '%s'", prot);
+    complain(_("Unsupported URL format '%s'"), prot);
   g_free(prot);
   g_free(host);
   g_free(path);
   if (downloadDone)
     (*downloadDone)();
+  busy=0;
   return res;
 }
 
 /* convert a relative pathname to the full pathname, on the user's selected
  * server. */
 static char *remote_file(char *relative) {
-  char *server = gnome_config_get_string(
+  char *server, *ret;
+
+  if(access("/etc/redhat-release",0)==0)
+    server = gnome_config_get_string(
+     		"/gnorpm/rpmfind/server=http://www.redhat.com/RDF");
+  else
+    server = gnome_config_get_string(
 		"/gnorpm/rpmfind/server=http://rufus.w3.org/linux/RDF");
-  char *ret = g_strconcat(server, "/", relative, NULL);
+  ret = g_strconcat(server, "/", relative, NULL);
   g_free(server);
   return ret;
 }
@@ -234,6 +264,7 @@ static char *remote_file(char *relative) {
  * copy is too old, download a new version */
 char *update_file(char *filename) {
   char *localfile, *tmp;
+  char *workfile;
   /* maximum age of cached file in seconds */
   int max_age = gnome_config_get_int("/gnorpm/rpmfind/expire=14") * 86400;
   struct stat info;
@@ -244,22 +275,43 @@ char *update_file(char *filename) {
   localfile = gnome_util_home_file(tmp);
   g_free(tmp);
 
+  tmp = g_strconcat("gnorpm.d/", filename, ".download", NULL);
+  workfile = gnome_util_home_file(tmp);
+  g_free(tmp);
+
+  unlink(workfile);
+
   /* if local file doesn't exist, or is too old, reget it. */
   if (!g_file_exists(localfile) || 
       (!stat(localfile, &info) && info.st_mtime + max_age < time(NULL))) {
-    FILE *fd = fopen(localfile, "wb");
+    FILE *fd = fopen(workfile, "wb");
     char *url = remote_file(filename);
-    int len = url_get_to_file(url, fd);
+    int len;
+        
+    /*printf("Getting '%s'\n", url);*/
+    
+    len = url_get_to_file(url, fd);
 
-    g_free(url);
     fclose(fd);
+    g_free(url);
     if (len <= 0) {
-      g_warning("Couldn't get data at %s", url);
+      complain(_("Couldn't get data"));
       unlink(localfile);
       g_free(localfile);
+      g_free(workfile);
       return NULL;
     }
+    if(rename(workfile,localfile)<0)
+    {
+	complain(_("Couldn't rename workfile to '%s'"), localfile);
+	unlink(localfile);
+        g_free(workfile);
+	g_free(localfile);
+	return NULL;
+    }
+    
   }
+  g_free(workfile);
   return localfile;
 }
 
@@ -276,7 +328,7 @@ char *url_get_to_temp(char *relative) {
   fclose(fd);
   
   if (len <= 0) {
-    g_warning("Couldn't get data at %s", url);
+    complain(_("Couldn't get data"));
     unlink(template);
     g_free(template);
     return NULL;
@@ -291,15 +343,15 @@ char *url_download_file(char *url) {
   int len;
 
   if (!g_file_exists(dir) && mkdir(dir, 0755))
-    g_warning("couldn't create directory %s", dir);
+    complain(_("couldn't create directory %s"), dir);
 
-  filename = g_strconcat(dir, "/", g_basename(url), NULL);
+  filename = g_strconcat(dir, "/", g_basename(url), ".tmp", NULL);
   g_free(dir);
   fp = fopen(filename, "wb");
   len = url_get_to_file(url, fp);
   fclose(fp);
   if (len <= 0) {
-    g_warning("Couldn't get data at %s", url);
+    complain(_("Couldn't get data at %s"), url);
     unlink(filename);
     g_free(filename);
     return NULL;
@@ -313,16 +365,16 @@ void cache_create_dirs(void) {
 
   dir = gnome_util_home_file("gnorpm.d");
   if (!g_file_exists(dir) && mkdir(dir, 0755))
-    g_warning("couldn't create directory %s", dir);
+    complain(_("Couldn't create directory %s"), dir);
   g_free(dir);
 
   dir = gnome_util_home_file("gnorpm.d/resources");
   if (!g_file_exists(dir) && mkdir(dir, 0755))
-    g_warning("couldn't create directory %s", dir);
+    complain(_("Couldn't create directory %s"), dir);
   g_free(dir);
   dir = gnome_util_home_file("gnorpm.d/resources/distribs");
   if (!g_file_exists(dir) && mkdir(dir, 0755))
-    g_warning("couldn't create directory %s", dir);
+    complain(_("Couldn't create directory %s"), dir);
   g_free(dir);
 
   /* make sure we have the rpm-dir config entry set up */
