@@ -208,9 +208,10 @@ static void rpm_dentry_edit_select_package(GtkWidget *w, RpmDentryEdit *self) {
   headerGetEntry(h, RPMTAG_OLDFILENAMES, NULL, (void**)&strlist, &len);
   if(len==0)
   {
-	  headerGetEntry(h, RPMTAG_BASENAMES, NULL, (void**)&strlist, &len);
-	  headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void**)&dirnames, &len);
-	  headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void**)&dirindex, &len);
+    gint32 tmp;
+    headerGetEntry(h, RPMTAG_BASENAMES, NULL, (void**)&strlist, &len);
+    headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void**)&dirnames, &tmp);
+    headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void**)&dirindex, &tmp);
   }
   headerGetEntry(h, RPMTAG_FILESIZES, NULL, (void**)&sizes, &len);
   headerGetEntry(h, RPMTAG_FILEMODES, NULL, (void**)&modes, &len);
