@@ -23,6 +23,7 @@
 #include "rpmprops.h"
 #include <rpmlib.h>
 #include <string.h>
+#include <glib.h>
 #include "misc.h"
 #include "install.h"
 
@@ -37,6 +38,7 @@ static void rpm_props_box_apply(GnomePropertyBox *self, gint page_num);
 
 static void rpm_props_box_class_init(RpmPropsBoxClass *klass);
 static void rpm_props_box_init(RpmPropsBox *box);
+static gchar* adjust_proxy_text(const gchar *proxyText);
 
 extern GnomeApp *gnorpm_app;
 
@@ -981,6 +983,28 @@ static void rpm_props_dist_changed(GtkButton *button, RpmPropsBox *self) {
   gnome_property_box_changed(GNOME_PROPERTY_BOX(self));
 }
 
+static gchar* adjust_proxy_text(const gchar *proxyText) {
+  gchar *text, *result;
+
+  result = g_strchug(g_strdup(proxyText));
+  if (!strlen(result))
+    return result;
+
+  if (!strstr(result, "http://")) {
+    text = g_strconcat("http://", result, NULL);
+    g_free(result);
+    result = g_strdup(text);
+    g_free(text);
+  }
+  if (!strstr(result + 7, ":")) {
+    /* Left off the proxy port, so insert a reasonable guess (Squid) */
+    text = g_strconcat(result, ":3128", NULL);
+    g_free(result);
+    result = g_strdup(text);
+    g_free(text);
+  }
+  return result;
+}
 #endif
 
 static void rpm_props_box_destroy(GtkObject *object) {
@@ -996,6 +1020,8 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
   gint i, rows;
   gushort r,g,b,a;
   gchar buf[1024], *text, **list;
+  gchar *httpStr = NULL;
+  gchar *ftpStr = NULL;
 
   switch (page_num) {
   case RPM_PROPS_FLAGS:
@@ -1043,10 +1069,14 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
     break;
 #ifdef WITH_RPMFIND
   case RPM_PROPS_NETWORK:
-    gnome_config_set_string("/gnorpm/rpmfind/http-proxy",
-			    gtk_entry_get_text(GTK_ENTRY(self->httpProxy)));
-    gnome_config_set_string("/gnorpm/rpmfind/ftp-proxy",
-			    gtk_entry_get_text(GTK_ENTRY(self->ftpProxy)));
+    text = gtk_editable_get_chars(GTK_EDITABLE(self->httpProxy), 0, -1);
+    httpStr = adjust_proxy_text(text);
+    g_free(text);
+    gnome_config_set_string("/gnorpm/rpmfind/http-proxy", httpStr);
+    text = gtk_editable_get_chars(GTK_EDITABLE(self->ftpProxy), 0, -1);
+    ftpStr = adjust_proxy_text(text);
+    g_free(text);
+    gnome_config_set_string("/gnorpm/rpmfind/ftp-proxy", ftpStr);
     gnome_config_set_string("/gnorpm/rpmfind/proxy-user",
 			    gtk_entry_get_text(GTK_ENTRY(self->proxyUser)));
     text = g_strdup(gtk_entry_get_text(GTK_ENTRY(self->proxyPass)));
@@ -1109,6 +1139,9 @@ static void rpm_props_box_apply(GnomePropertyBox *object, gint page_num) {
   gnome_config_sync();
   if (parent_class->apply)
     (* parent_class->apply)(object, page_num);
+
+  g_free(httpStr);
+  g_free(ftpStr);
 }
 
 #ifdef HAVE_RPM_2_5
