@@ -26,10 +26,13 @@
 GtkWidget *verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
   Header h;
   GtkWidget *win, *vbox, *pkg_name, *fname, *swin, *clist, *button;
-  gchar buf1[512], buf2[512], *s1, *s2, *s3, **files;
+  gchar buf1[512], buf2[512], buf3[512], *s1, *s2, *s3, **files;
   gchar *headings[] = {N_("Package"), N_("File"), N_("Problem")}, *row[3];
   guint index, i, count, failCount = 0;
   gint result;
+  gint32 *dirindex=NULL;
+  gchar **dirnames=NULL;
+  gint32 len;
 
   headings[0] = _(headings[0]);
   headings[1] = _(headings[1]);
@@ -98,6 +101,12 @@ GtkWidget *verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
     g_snprintf(buf1, 511, "%s-%s-%s", s1, s2, s3);
     gtk_label_set(GTK_LABEL(pkg_name), buf1);
     headerGetEntry(h, RPMTAG_OLDFILENAMES, NULL, (void **)&files, &count);
+    if (files == 0)
+    {
+      headerGetEntry(h, RPMTAG_BASENAMES, NULL, (void**)&files, &count);
+      headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void**)&dirnames, &len);
+      headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void**)&dirindex, &len);
+    }
     if (files) {
       for (i = 0; i < count; i++) {
 	gtk_label_set(GTK_LABEL(fname), files[i]);
@@ -105,10 +114,16 @@ GtkWidget *verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
 	  gtk_main_iteration();
 	if (rpmVerifyFile(hdl->root, h, i, &result, omitMask) || result) {
 	  row[0] = buf1;
-	  row[1] = files[i];
+	  if(dirnames)
+	  {
+	    snprintf(buf3, 512, "%s%s", dirnames[dirindex[i]], files[i]);
+	    row[1]=buf3;
+	  }
+	  else
+	    row[1] = files[i];
+	  
 	  row[2] = &buf2[2];
 	  buf2[0] = '\0';
-	  buf2[2] = '\0';
 	  if (!result) row[2] = _("missing");
 	  else {
 	    if (result & RPMVERIFY_MD5)      strcat(buf2, _(", md5"));
